// TODO : pacco cli

use argh::FromArgs;

#[derive(FromArgs, PartialEq, Debug)]
/// Pacco Package Server
pub struct PaccoCLI {
    #[argh(subcommand)]
    pub cmd: PaccoCLICommands,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "pkg")]
/// Package commands
pub struct PackageCommand {
    #[argh(subcommand)]
    pub cmd: PackageCommands,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "repo")]
/// Repository commands
pub struct RepoCommand {
    #[argh(subcommand)]
    pub cmd: RepoCommands,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand)]
pub enum PaccoCLICommands {
    Serve(ServeCommand),
    Build(BuildCommand),
    Package(PackageCommand),
    Repo(RepoCommand),
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "serve", description = "Start a package server")]
pub struct ServeCommand {
    #[argh(option)]
    /// config file
    pub config: Option<String>,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "build", description = "Build a package")]
pub struct BuildCommand {
    #[argh(option)]
    /// push to a repo after building
    pub push: Option<String>,

    #[argh(switch)]
    /// build is run inside a CI environment
    pub ci: bool,

    #[argh(option)]
    /// output directory
    pub out: Option<String>,

    #[argh(option, default = r#"String::from("archlinux")"#)]
    /// docker build image
    pub image: String,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand)]
pub enum PackageCommands {
    Init(PackageInitCommand),
    Info(PackageInfoCommand),
    Push(PackagePushCommand),
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "init")]
/// Init a template PKGBUILD
pub struct PackageInitCommand {}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "info")]
/// Print package info from PKGBUILD
pub struct PackageInfoCommand {}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "push")]
/// Push a package file to a pacco server
pub struct PackagePushCommand {
    #[argh(switch)]
    /// push signature
    pub signed: bool,
    #[argh(positional)]
    /// package file
    pub package: String,
    #[argh(positional)]
    /// repository
    pub repo: String,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand)]
pub enum RepoCommands {
    Add(RepoAdd),
    Rm(RepoRm),
    Init(RepoInit),
    Index(RepoIndex),
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "add")]
/// Add a package to repo
pub struct RepoAdd {
    #[argh(positional)]
    /// package to add
    pub package: String,
    #[argh(positional)]
    /// repository
    pub repo: Option<String>,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "rm")]
/// Remove a package from repo
pub struct RepoRm {
    #[argh(positional)]
    /// package to remove
    pub package: String,
    #[argh(positional)]
    /// repository
    pub repo: Option<String>,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "init")]
/// Init a new repo
pub struct RepoInit {
    #[argh(option, short = 'a')]
    /// arch
    pub arch: Vec<String>,

    #[argh(positional)]
    /// repo name
    pub name: String,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "index")]
/// Reindex all packages in repo
pub struct RepoIndex {
    #[argh(positional)]
    /// repository
    pub repo: Option<String>,
}

// TODO : new cli cmd

// conf through env vars -> build container

// TODO : move serving fn to pacco serve <config>
