#![allow(non_camel_case_types)]
#[derive(Debug, Clone, PartialEq, Eq)]
pub enum Architecture {
    x86_64,
    aarch64,
    any,
}

impl Architecture {
    pub fn parse(val: &str) -> Option<Architecture> {
        match val {
            "x86_64" => Some(Architecture::x86_64),
            "aarch64" => Some(Architecture::aarch64),
            "any" => Some(Architecture::any),
            _ => None,
        }
    }

    /// The architecture of the binary host
    pub fn own() -> Architecture {
        #[cfg(target_arch = "x86_64")]
        {
            return Architecture::x86_64;
        }
        #[cfg(target_arch = "aarch64")]
        {
            return Architecture::aarch64;
        }
    }

    pub fn to_string(&self) -> String {
        match self {
            Architecture::x86_64 => "x86_64".to_string(),
            Architecture::aarch64 => "aarch64".to_string(),
            Architecture::any => "any".to_string(),
        }
    }
}
